--------------------------------------------------------------------------------
-- Ball module -----------------------------------------------------------------
--------------------------------------------------------------------------------

Ball = {}

function Ball.new()
	local _self = {}
	
	-- Private variables:
	local _x -- Ball horizontal position
	local _y -- Ball vertical position
	local _radius -- Ball radius
	local _vSpeed -- Ball speed
	local _hSpeed -- x salt
	--[[ if _vMove is true the ball have to move up
		else the ball have to move down ]]--
	local _vMove
	--[[ if _hMove is true the ball have to move up
		else the ball have to move down ]]--
	local _hMove
	
	-- Private variables:
	-- TODO: add some private variables?
	
	-- Private methods:
	function moveUp(t)
		_y = _y - (_vSpeed * 100 * t)
	end
	
	function moveDown(t)
		_y = _y + (_vSpeed * 100 * t)
	end
	
	function moveRight(t)
		_x = _x + (_hSpeed * t)
	end
	
	function moveLeft(t)
		_x = _x - (_hSpeed * t)
	end
	
	-- Source and explanation:
	-- http://stackoverflow.com/questions/401847/circle-rectangle-collision-detection-intersection
	function ballIntersect(cx, cy, cr, rx, ry, rw, rh)
		local circle_distance_x = math.abs(cx - rx - rw/2)
		local circle_distance_y = math.abs(cy - ry - rh/2)
		
		if circle_distance_x > (rw/2 + cr) then return false end
		if circle_distance_y > (rh/2 + cr) then return false end
		
		if circle_distance_x <= (rw/2) then return true end
		if circle_distance_y <= (rh/2) then return true end
		
		local corner_distance_sq = math.pow(circle_distance_x - rw/2, 2) + math.pow(circle_distance_y - rh/2, 2)
		
		return corner_distance_sq <= math.pow(cr, 2)
	end
	
	function salt()
		local rand = math.random (0, 1)
		if rand == 1 then
			_xMove = true
		else
			_xMove = false
		end
		_hSpeed = math.random (1, _vSpeed) * math.random (50, 100)
	end
	
	function wallBump()
		_hSpeed = math.random (1, _vSpeed) * math.random (50, 100)
	end
	
	function move(t)
		if _vMove == true then
			moveUp(t)
		else
			moveDown(t)
		end
		if _hMove == true then
			moveRight(t)
		else
			moveLeft(t)
		end
	end
	
	-- Public methods:
	-- init method:
	function _self.init(x, y, radius, speed, direction, vMove)
		_x = x
		_y = y
		_radius = radius
		_vSpeed = speed
		_vMove = vMove
		_hSpeed = _vSpeed * 100
	end
	
	-- update method:
	function _self.update(t, player)
		-- top wall
		if ballIntersect(
			_x,
			_y,
			_radius,
			0, -1,
			love.graphics.getWidth(), 1
		) == true then
			_vMove = false
			wallBump()
		end
		-- left wall
		if ballIntersect(
			_x,
			_y,
			_radius, -1,
			0, 1, love.graphics.getHeight()
		) == true then
			_hMove = true
			wallBump()
		end
		-- right wall
		if ballIntersect(
			_x,
			_y,
			_radius,
			love.graphics.getWidth()+1, 0,
			1, love.graphics.getHeight()
		) == true then
			_hMove = false
			wallBump()
		end
		-- player
		if ballIntersect(
			_x,
			_y,
			_radius,
			player.getX(), player.getY(),
			player.getWidth(), player.getHeight()
		) == true then
			_vMove = true
			wallBump()
		end
		-- bot wall
		if ballIntersect(
			_x,
			_y,
			_radius,
			0, love.graphics.getHeight()+10,
			love.graphics.getWidth(), 10
		) == true then
			return false
		end
		
		move(t)
		
		return true
	end
	
	-- draw method:
	function _self.draw()
		love.graphics.setColor(255, 255, 255)
		love.graphics.circle(
			"fill",
			_x,
			_y,
			_radius
		)
	end
	
	return _self
end